@@ -16,23 +16,18 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef TYPEDEFS_H
 #define TYPEDEFS_H
 
+#include <glib.h>
+#include <glib-object.h>
+
 #define MEGABYTE (1024 * 1024)
 
-#define RC_DIR              ".gnome2/file-roller"
-#define RC_BOOKMARKS_FILE   ".gnome2/file-roller/bookmarks"
-#define RC_RECENT_FILE      ".gnome2/file-roller/recents"
-#define RC_OPTIONS_DIR      ".gnome2/file-roller/options"
-
-#define OLD_RC_BOOKMARKS_FILE   ".file-roller/bookmarks"
-#define OLD_RC_RECENT_FILE      ".file-roller/recents"
-#define OLD_RC_OPTIONS_DIR      ".file-roller/options"
+#define ADD_FOLDER_OPTIONS_DIR  "file-roller/options"
 
 typedef enum { /*< skip >*/
 	FR_WINDOW_SORT_BY_NAME = 0,
